require "database_cleaner/octopus_sharding/version"

module DatabaseCleaner
  module OctopusSharding
    extend ActiveSupport::Concern

    included do
      alias_method_chain :start, :octopus
      alias_method_chain :clean, :octopus
    end

    def start_with_octopus
      if ::Octopus.enabled?
        each_shard do
          start_without_octopus
        end
      else
        start_without_octopus
      end
    end

    def clean_with_octopus
      if ::Octopus.enabled?
        each_shard do
          clean_without_octopus
        end
      else
        clean_without_octopus
      end
    end

    private

    def each_shard
      ::ActiveRecord::Base.connection.instance_variable_get('@shards').each do |shard_name, _conn|
        Octopus.using(shard_name) do
          yield
        end
      end
    end
  end
end

DatabaseCleaner::Base.__send__ :include, DatabaseCleaner::OctopusSharding
